/**
 * Copyright &copy; 2012-2016 <a href="https://github.com/thinkgem/jeesite">JeeSite</a> All rights reserved.
 */
package com.wechat.global.base.interceptor;

import java.util.ArrayList;
import java.util.List;
import java.util.regex.Pattern;

import com.fasterxml.jackson.annotation.JsonIgnore;

/**
 * 分页类
 * @author ThinkGem
 * @version 2013-7-2
 * @param <T>
 */
public class Page<T> {
	
	private int currentPage = 1; // 当前页码
	private int pageSize = 10; // 页面大小，设置为“-1”表示不进行分页（分页无效）
	
	private long totalSize;// 总记录数，设置为“-1”表示不查询总数
	
	private int first;// 首页索引
	private int last;// 尾页索引
	private int prevPage;// 上一页索引
	private int nextPage;// 下一页索引
	
	private boolean firstPage;//是否是第一页
	private boolean lastPage;//是否是最后一页

	private int length = 8;// 显示页面长度
	private int slider = 1;// 前后显示页面长度
	
	private List<T> resultList = new ArrayList<T>();
	
	private String orderBy = ""; // 标准查询有效， 实例： updatedate desc, name asc

	private String funcName = "page"; // 设置点击页码调用的js函数名称，默认为page，在一页有多个分页对象时使用。
	
	private String funcParam = ""; // 函数的附加参数，第三个参数值。
	

	public Page() {
		this.pageSize = -1;
	}
	
	/**
	 * 构造方法
	 * @param currentPage 当前页码
	 * @param pageSize 分页大小
	 */
	public Page(int currentPage, int pageSize) {
		this(currentPage, pageSize, 0);
	}
	
	/**
	 * 构造方法
	 * @param currentPage 当前页码
	 * @param pageSize 分页大小
	 * @param totalSize 数据条数
	 */
	public Page(int currentPage, int pageSize, long totalSize) {
		this(currentPage, pageSize, totalSize, new ArrayList<T>());
	}
	
	/**
	 * 构造方法
	 * @param currentPage 当前页码
	 * @param pageSize 分页大小
	 * @param totalSize 数据条数
	 * @param resultList 本页数据对象列表
	 */
	public Page(int currentPage, int pageSize, long totalSize, List<T> resultList) {
		this.setTotalSize(totalSize);
		this.setCurrentPage(currentPage);
		this.pageSize = pageSize;
		this.resultList = resultList;
	}
	
	/**
	 * 初始化参数
	 */
	public void initialize(){
				
		//1
		this.first = 1;
		
		this.last = (int)(totalSize / (this.pageSize < 1 ? 20 : this.pageSize) + first - 1);
		
		if (this.totalSize % this.pageSize != 0 || this.last == 0) {
			this.last++;
		}

		if (this.last < this.first) {
			this.last = this.first;
		}
		
		if (this.currentPage <= 1) {
			this.currentPage = this.first;
			this.firstPage=true;
		}

		if (this.currentPage >= this.last) {
			this.currentPage = this.last;
			this.lastPage=true;
		}

		if (this.currentPage < this.last - 1) {
			this.nextPage = this.currentPage + 1;
		} else {
			this.nextPage = this.last;
		}

		if (this.currentPage > 1) {
			this.prevPage = this.currentPage - 1;
		} else {
			this.prevPage = this.first;
		}
		
		//2
		if (this.currentPage < this.first) {// 如果当前页小于首页
			this.currentPage = this.first;
		}

		if (this.currentPage > this.last) {// 如果当前页大于尾页
			this.currentPage = this.last;
		}
		
	}


	/**
	 * 获取设置总数
	 * @return
	 */
	public long getTotalSize() {
		return totalSize;
	}

	/**
	 * 设置数据总数
	 * @param totalSize
	 */
	public void setTotalSize(long totalSize) {
		this.totalSize = totalSize;
		if (pageSize >= totalSize){
			currentPage = 1;
		}
	}
	
	/**
	 * 获取当前页码
	 * @return
	 */
	public int getCurrentPage() {
		return currentPage;
	}
	
	/**
	 * 设置当前页码
	 * @param currentPage
	 */
	public void setCurrentPage(int currentPage) {
		this.currentPage = currentPage;
	}
	
	/**
	 * 获取页面大小
	 * @return
	 */
	public int getPageSize() {
		return pageSize;
	}

	/**
	 * 设置页面大小（最大500）
	 * @param pageSize
	 */
	public void setPageSize(int pageSize) {
		this.pageSize = pageSize <= 0 ? 10 : pageSize;// > 500 ? 500 : pageSize;
	}

	/**
	 * 首页索引
	 * @return
	 */
	@JsonIgnore
	public int getFirst() {
		return first;
	}

	/**
	 * 尾页索引
	 * @return
	 */
	@JsonIgnore
	public int getLast() {
		return last;
	}
	
	/**
	 * 获取页面总数
	 * @return getLast();
	 */
	@JsonIgnore
	public int getTotalPage() {
		return getLast();
	}

	/**
	 * 是否为第一页
	 * @return
	 */
	@JsonIgnore
	public boolean isFirstPage() {
		return firstPage;
	}

	/**
	 * 是否为最后一页
	 * @return
	 */
	@JsonIgnore
	public boolean isLastPage() {
		return lastPage;
	}
	
	/**
	 * 上一页索引值
	 * @return
	 */
	@JsonIgnore
	public int getPrevPage() {
		if (isFirstPage()) {
			return currentPage;
		} else {
			return currentPage - 1;
		}
	}

	/**
	 * 下一页索引值
	 * @return
	 */
	@JsonIgnore
	public int getNext() {
		if (isLastPage()) {
			return currentPage;
		} else {
			return currentPage + 1;
		}
	}
	
	/**
	 * 获取本页数据对象列表
	 * @return resultList<T>
	 */
	public List<T> getResultList() {
		return resultList;
	}

	/**
	 * 设置本页数据对象列表
	 * @param list
	 */
	public Page<T> setResultList(List<T> resultList) {
		this.resultList = resultList;
		initialize();
		return this;
	}

	/**
	 * 获取查询排序字符串
	 * @return
	 */
	@JsonIgnore
	public String getOrderBy() {
		// SQL过滤，防止注入 
		String reg = "(?:')|(?:--)|(/\\*(?:.|[\\n\\r])*?\\*/)|"
					+ "(\\b(select|update|and|or|delete|insert|trancate|char|into|substr|ascii|declare|exec|count|master|into|drop|execute)\\b)";
		Pattern sqlPattern = Pattern.compile(reg, Pattern.CASE_INSENSITIVE);
		if (sqlPattern.matcher(orderBy).find()) {
			return "";
		}
		return orderBy;
	}

	/**
	 * 设置查询排序，标准查询有效， 实例： updatedate desc, name asc
	 */
	public void setOrderBy(String orderBy) {
		this.orderBy = orderBy;
	}

	/**
	 * 获取点击页码调用的js函数名称
	 * function ${page.funcName}(currentPage){location="${ctx}/list-${category.id}${urlSuffix}?currentPage="+i;}
	 * @return
	 */
	@JsonIgnore
	public String getFuncName() {
		return funcName;
	}

	/**
	 * 设置点击页码调用的js函数名称，默认为page，在一页有多个分页对象时使用。
	 * @param funcName 默认为page
	 */
	public void setFuncName(String funcName) {
		this.funcName = funcName;
	}

	/**
	 * 获取分页函数的附加参数
	 * @return
	 */
	@JsonIgnore
	public String getFuncParam() {
		return funcParam;
	}

	/**
	 * 设置分页函数的附加参数
	 * @return
	 */
	public void setFuncParam(String funcParam) {
		this.funcParam = funcParam;
	}

	
	/**
	 * 分页是否有效
	 * @return this.pageSize==-1
	 */
	@JsonIgnore
	public boolean isDisabled() {
		return this.pageSize==-1;
	}
	
	/**
	 * 是否进行总数统计
	 * @return this.totalSize==-1
	 */
	@JsonIgnore
	public boolean isNotTotalSize() {
		return this.totalSize==-1;
	}
	
	/**
	 * 获取 Hibernate FirstResult
	 */
	public int getFirstResult(){
		int firstResult = (getCurrentPage() - 1) * getPageSize();
		if (firstResult >= getTotalSize()) {
			firstResult = 0;
		}
		return firstResult;
	}
	/**
	 * 获取 Hibernate MaxResults
	 */
	public int getMaxResults(){
		return getPageSize();
	}

	
}
